@@ -23,7 +23,7 @@
 /**
  * SECTION:completionwords
  * @title: GtkSourceCompletionWords
- * @short_description: A #GtkSourceCompletionProvider for the completion of words
+ * @short_description: A GtkSourceCompletionProvider for the completion of words
  *
  * The #GtkSourceCompletionWords is an example of an implementation of
  * the #GtkSourceCompletionProvider interface. The proposals are words
@@ -217,7 +217,11 @@ gtk_source_completion_words_populate (GtkSourceCompletionProvider *provider,
 	GtkTextIter iter;
 	gchar *word;
 
-	gtk_source_completion_context_get_iter (context, &iter);
+	if (!gtk_source_completion_context_get_iter (context, &iter))
+	{
+		gtk_source_completion_context_add_proposals (context, provider, NULL, TRUE);
+		return;
+	}
 
 	g_free (words->priv->word);
 	words->priv->word = NULL;
@@ -231,10 +235,7 @@ gtk_source_completion_words_populate (GtkSourceCompletionProvider *provider,
 	     g_utf8_strlen (word, -1) < words->priv->minimum_word_size))
 	{
 		g_free (word);
-		gtk_source_completion_context_add_proposals (context,
-		                                             provider,
-		                                             NULL,
-		                                             TRUE);
+		gtk_source_completion_context_add_proposals (context, provider, NULL, TRUE);
 		return;
 	}
 
@@ -426,24 +427,24 @@ gtk_source_completion_words_class_init (GtkSourceCompletionWordsClass *klass)
 	g_object_class_install_property (object_class,
 	                                 PROP_NAME,
 	                                 g_param_spec_string ("name",
-	                                                      C_("Object", "Name"),
-	                                                      _("The provider name"),
+	                                                      "Name",
+	                                                      "The provider name",
 	                                                      NULL,
 	                                                      G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 
 	g_object_class_install_property (object_class,
 	                                 PROP_ICON,
 	                                 g_param_spec_object ("icon",
-	                                                      _("Icon"),
-	                                                      _("The provider icon"),
+	                                                      "Icon",
+	                                                      "The provider icon",
 	                                                      GDK_TYPE_PIXBUF,
 	                                                      G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 
 	g_object_class_install_property (object_class,
 	                                 PROP_PROPOSALS_BATCH_SIZE,
 	                                 g_param_spec_uint ("proposals-batch-size",
-	                                                    _("Proposals Batch Size"),
-	                                                    _("Number of proposals added in one batch"),
+	                                                    "Proposals Batch Size",
+	                                                    "Number of proposals added in one batch",
 	                                                    1,
 	                                                    G_MAXUINT,
 	                                                    300,
@@ -452,8 +453,8 @@ gtk_source_completion_words_class_init (GtkSourceCompletionWordsClass *klass)
 	g_object_class_install_property (object_class,
 	                                 PROP_SCAN_BATCH_SIZE,
 	                                 g_param_spec_uint ("scan-batch-size",
-	                                                    _("Scan Batch Size"),
-	                                                    _("Number of lines scanned in one batch"),
+	                                                    "Scan Batch Size",
+	                                                    "Number of lines scanned in one batch",
 	                                                    1,
 	                                                    G_MAXUINT,
 	                                                    50,
@@ -462,8 +463,8 @@ gtk_source_completion_words_class_init (GtkSourceCompletionWordsClass *klass)
 	g_object_class_install_property (object_class,
 	                                 PROP_MINIMUM_WORD_SIZE,
 	                                 g_param_spec_uint ("minimum-word-size",
-	                                                    _("Minimum Word Size"),
-	                                                    _("The minimum word size to complete"),
+	                                                    "Minimum Word Size",
+	                                                    "The minimum word size to complete",
 	                                                    2,
 	                                                    G_MAXUINT,
 	                                                    2,
@@ -472,8 +473,8 @@ gtk_source_completion_words_class_init (GtkSourceCompletionWordsClass *klass)
 	g_object_class_install_property (object_class,
 	                                 PROP_INTERACTIVE_DELAY,
 	                                 g_param_spec_int ("interactive-delay",
-	                                                   _("Interactive Delay"),
-	                                                   _("The delay before initiating interactive completion"),
+	                                                   "Interactive Delay",
+	                                                   "The delay before initiating interactive completion",
 	                                                   -1,
 	                                                   G_MAXINT,
 	                                                   50,
@@ -482,8 +483,8 @@ gtk_source_completion_words_class_init (GtkSourceCompletionWordsClass *klass)
 	g_object_class_install_property (object_class,
 	                                 PROP_PRIORITY,
 	                                 g_param_spec_int ("priority",
-	                                                   _("Priority"),
-	                                                   _("Provider priority"),
+	                                                   "Priority",
+	                                                   "Provider priority",
 	                                                   G_MININT,
 	                                                   G_MAXINT,
 	                                                   0,
@@ -499,8 +500,8 @@ gtk_source_completion_words_class_init (GtkSourceCompletionWordsClass *klass)
 	g_object_class_install_property (object_class,
 	                                 PROP_ACTIVATION,
 					 g_param_spec_flags ("activation",
-							     _("Activation"),
-							     _("The type of activation"),
+							     "Activation",
+							     "The type of activation",
 							     GTK_SOURCE_TYPE_COMPLETION_ACTIVATION,
 							     GTK_SOURCE_COMPLETION_ACTIVATION_INTERACTIVE |
 							     GTK_SOURCE_COMPLETION_ACTIVATION_USER_REQUESTED,
@@ -516,7 +517,10 @@ gtk_source_completion_words_get_start_iter (GtkSourceCompletionProvider *provide
 	gchar *word;
 	glong nb_chars;
 
-	gtk_source_completion_context_get_iter (context, iter);
+	if (!gtk_source_completion_context_get_iter (context, iter))
+	{
+		return FALSE;
+	}
 
 	word = get_word_at_iter (iter);
 	g_return_val_if_fail (word != NULL, FALSE);
