@@ -43,6 +43,18 @@
  * A file loader should be used only for one load operation, including errors
  * handling. If an error occurs, you can reconfigure the loader and relaunch the
  * operation with gtk_source_file_loader_load_async().
+ *
+ * Running a #GtkSourceFileLoader is an undoable action for the
+ * #GtkSourceBuffer. That is, gtk_source_buffer_begin_not_undoable_action() and
+ * gtk_source_buffer_end_not_undoable_action() are called, which delete the
+ * undo/redo history.
+ *
+ * After a file loading, the buffer is reset to the contents provided by the
+ * #GFile or #GInputStream, so the buffer is set as “unmodified”, that is,
+ * gtk_text_buffer_set_modified() is called with %FALSE. If the contents isn't
+ * saved somewhere (for example if you load from stdin), then you should
+ * probably call gtk_text_buffer_set_modified() with %TRUE after calling
+ * gtk_source_file_loader_load_finish().
  */
 
 #if 0
@@ -273,7 +285,7 @@ set_default_candidate_encodings (GtkSourceFileLoader *loader)
 	 * GtkSourceFile's encoding has been set by a FileLoader or FileSaver,
 	 * put it at the beginning of the list.
 	 */
-	list = _gtk_source_encoding_get_default_candidates ();
+	list = gtk_source_encoding_get_default_candidates ();
 
 	if (loader->priv->file == NULL)
 	{
@@ -394,7 +406,7 @@ gtk_source_file_loader_class_init (GtkSourceFileLoaderClass *klass)
 	 */
 	g_object_class_install_property (object_class, PROP_LOCATION,
 					 g_param_spec_object ("location",
-							      _("Location"),
+							      "Location",
 							      "",
 							      G_TYPE_FILE,
 							      G_PARAM_READWRITE |
@@ -411,7 +423,7 @@ gtk_source_file_loader_class_init (GtkSourceFileLoaderClass *klass)
 	 */
 	g_object_class_install_property (object_class, PROP_INPUT_STREAM,
 					 g_param_spec_object ("input-stream",
-							      _("Input stream"),
+							      "Input stream",
 							      "",
 							      G_TYPE_INPUT_STREAM,
 							      G_PARAM_READWRITE |
@@ -661,6 +673,9 @@ create_input_stream (GtkSourceFileLoader *loader)
 			case GTK_SOURCE_COMPRESSION_TYPE_NONE:
 				/* NOOP */
 				break;
+
+			default:
+				g_assert_not_reached ();
 		}
 	}
 
@@ -885,11 +900,11 @@ gtk_source_file_loader_new_from_stream (GtkSourceBuffer *buffer,
  * For convenience, @candidate_encodings can contain duplicates. Only the first
  * occurrence of a duplicated encoding is kept in the list.
  *
- * By default the candidate encodings are (in that order):
- * 1. If set, the #GtkSourceFile's encoding. See gtk_source_file_get_encoding().
- * 2. Depending on the current locale (language and country), a list of common
- * encodings are added. The UTF-8 encoding and the current locale encoding are
- * always present.
+ * By default the candidate encodings are (in that order in the list):
+ * 1. If set, the #GtkSourceFile's encoding as returned by
+ * gtk_source_file_get_encoding().
+ * 2. The default candidates as returned by
+ * gtk_source_encoding_get_default_candidates().
  *
  * Since: 3.14
  */
@@ -940,8 +955,8 @@ gtk_source_file_loader_get_file (GtkSourceFileLoader *loader)
  * gtk_source_file_loader_get_location:
  * @loader: a #GtkSourceFileLoader.
  *
- * Returns: (transfer none): the #GFile to load, or %NULL if an input stream is
- *   used.
+ * Returns: (nullable) (transfer none): the #GFile to load, or %NULL
+ * if an input stream is used.
  * Since: 3.14
  */
 GFile *
@@ -956,8 +971,8 @@ gtk_source_file_loader_get_location (GtkSourceFileLoader *loader)
  * gtk_source_file_loader_get_input_stream:
  * @loader: a #GtkSourceFileLoader.
  *
- * Returns: (transfer none): the #GInputStream to load, or %NULL if a #GFile is
- *   used.
+ * Returns: (nullable) (transfer none): the #GInputStream to load, or %NULL
+ * if a #GFile is used.
  * Since: 3.14
  */
 GInputStream *
@@ -1090,7 +1105,7 @@ gtk_source_file_loader_load_finish (GtkSourceFileLoader  *loader,
 				    GError              **error)
 {
 	gboolean ok;
-	gboolean update_buffer_properties;
+	gboolean update_file_properties;
 	GError *real_error = NULL;
 
 	g_return_val_if_fail (GTK_SOURCE_IS_FILE_LOADER (loader), FALSE);
@@ -1112,11 +1127,11 @@ gtk_source_file_loader_load_finish (GtkSourceFileLoader  *loader,
 	 * With the other errors, normally the contents hasn't been loaded into
 	 * the buffer, i.e. the buffer is still empty.
 	 */
-	update_buffer_properties = ok || (real_error != NULL &&
-					  real_error->domain == GTK_SOURCE_FILE_LOADER_ERROR &&
-					  real_error->code == GTK_SOURCE_FILE_LOADER_ERROR_CONVERSION_FALLBACK);
+	update_file_properties = ok || (real_error != NULL &&
+					real_error->domain == GTK_SOURCE_FILE_LOADER_ERROR &&
+					real_error->code == GTK_SOURCE_FILE_LOADER_ERROR_CONVERSION_FALLBACK);
 
-	if (update_buffer_properties && loader->priv->file != NULL)
+	if (update_file_properties && loader->priv->file != NULL)
 	{
 		/* The location is already updated at the beginning of the
 		 * operation.
@@ -1131,12 +1146,29 @@ gtk_source_file_loader_load_finish (GtkSourceFileLoader  *loader,
 		_gtk_source_file_set_compression_type (loader->priv->file,
 						       loader->priv->auto_detected_compression_type);
 
+		_gtk_source_file_set_externally_modified (loader->priv->file, FALSE);
+		_gtk_source_file_set_deleted (loader->priv->file, FALSE);
+
 		if (g_file_info_has_attribute (loader->priv->info, G_FILE_ATTRIBUTE_TIME_MODIFIED))
 		{
 			GTimeVal modification_time;
 			g_file_info_get_modification_time (loader->priv->info, &modification_time);
 			_gtk_source_file_set_modification_time (loader->priv->file, modification_time);
 		}
+
+		if (g_file_info_has_attribute (loader->priv->info, G_FILE_ATTRIBUTE_ACCESS_CAN_WRITE))
+		{
+			gboolean readonly;
+
+			readonly = !g_file_info_get_attribute_boolean (loader->priv->info,
+								       G_FILE_ATTRIBUTE_ACCESS_CAN_WRITE);
+
+			_gtk_source_file_set_readonly (loader->priv->file, readonly);
+		}
+		else
+		{
+			_gtk_source_file_set_readonly (loader->priv->file, FALSE);
+		}
 	}
 
 	reset (loader);
