@@ -22,9 +22,8 @@
 #ifndef __GTK_SOURCE_GUTTER_RENDERER_H__
 #define __GTK_SOURCE_GUTTER_RENDERER_H__
 
-#include <glib-object.h>
 #include <gtk/gtk.h>
-#include <gdk/gdk.h>
+#include <gtksourceview/gtksourcetypes.h>
 
 G_BEGIN_DECLS
 
@@ -36,13 +35,11 @@ G_BEGIN_DECLS
 #define GTK_SOURCE_IS_GUTTER_RENDERER_CLASS(klass)	(G_TYPE_CHECK_CLASS_TYPE ((klass), GTK_SOURCE_TYPE_GUTTER_RENDERER))
 #define GTK_SOURCE_GUTTER_RENDERER_GET_CLASS(obj)	(G_TYPE_INSTANCE_GET_CLASS ((obj), GTK_SOURCE_TYPE_GUTTER_RENDERER, GtkSourceGutterRendererClass))
 
-typedef struct _GtkSourceGutterRenderer		GtkSourceGutterRenderer;
 typedef struct _GtkSourceGutterRendererClass	GtkSourceGutterRendererClass;
 typedef struct _GtkSourceGutterRendererPrivate	GtkSourceGutterRendererPrivate;
 
 /**
  * GtkSourceGutterRendererState:
- *
  * @GTK_SOURCE_GUTTER_RENDERER_STATE_NORMAL: normal state
  * @GTK_SOURCE_GUTTER_RENDERER_STATE_CURSOR: area in the renderer represents the
  * line on which the insert cursor is currently positioned
@@ -59,6 +56,15 @@ typedef enum
 	GTK_SOURCE_GUTTER_RENDERER_STATE_SELECTED = 1 << 2
 } GtkSourceGutterRendererState;
 
+/**
+ * GtkSourceGutterRendererAlignmentMode:
+ * @GTK_SOURCE_GUTTER_RENDERER_ALIGNMENT_MODE_CELL: The full cell.
+ * @GTK_SOURCE_GUTTER_RENDERER_ALIGNMENT_MODE_FIRST: The first line.
+ * @GTK_SOURCE_GUTTER_RENDERER_ALIGNMENT_MODE_LAST: The last line.
+ *
+ * The alignment mode of the renderer, when a cell spans multiple lines (due to
+ * text wrapping).
+ **/
 typedef enum
 {
 	GTK_SOURCE_GUTTER_RENDERER_ALIGNMENT_MODE_CELL,
@@ -96,9 +102,23 @@ struct _GtkSourceGutterRendererClass
 
 	void (*end)                 (GtkSourceGutterRenderer      *renderer);
 
+	/**
+	 * GtkSourceGutterRendererClass::change_view:
+	 * @renderer: a #GtkSourceGutterRenderer.
+	 * @old_view: (allow-none): the old #GtkTextView.
+	 *
+	 * This is called when the text view changes for @renderer.
+	 */
 	void (*change_view)         (GtkSourceGutterRenderer      *renderer,
 	                             GtkTextView                  *old_view);
 
+	/**
+	 * GtkSourceGutterRendererClass::change_buffer:
+	 * @renderer: a #GtkSourceGutterRenderer.
+	 * @old_buffer: (allow-none): the old #GtkTextBuffer.
+	 *
+	 * This is called when the text buffer changes for @renderer.
+	 */
 	void (*change_buffer)       (GtkSourceGutterRenderer      *renderer,
 	                             GtkTextBuffer                *old_buffer);
 
