@@ -8,12 +8,12 @@
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2.1 of the License, or (at your option) any later version.
- * 
+ *
  * GtkSourceView is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
@@ -22,22 +22,13 @@
 #include "gtksourcegutterrendererlines.h"
 #include "gtksourceview.h"
 
-#define GTK_SOURCE_GUTTER_RENDERER_LINES_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE((object), GTK_SOURCE_TYPE_GUTTER_RENDERER_LINES, GtkSourceGutterRendererLinesPrivate))
-
 struct _GtkSourceGutterRendererLinesPrivate
 {
 	gint num_line_digits;
-
-	guint changed_handler_id;
+	gint prev_line_num;
 };
 
-G_DEFINE_TYPE (GtkSourceGutterRendererLines, gtk_source_gutter_renderer_lines, GTK_SOURCE_TYPE_GUTTER_RENDERER_TEXT)
-
-static void
-gtk_source_gutter_renderer_lines_finalize (GObject *object)
-{
-	G_OBJECT_CLASS (gtk_source_gutter_renderer_lines_parent_class)->finalize (object);
-}
+G_DEFINE_TYPE_WITH_PRIVATE (GtkSourceGutterRendererLines, gtk_source_gutter_renderer_lines, GTK_SOURCE_TYPE_GUTTER_RENDERER_TEXT)
 
 static GtkTextBuffer *
 get_buffer (GtkSourceGutterRendererLines *renderer)
@@ -46,7 +37,7 @@ get_buffer (GtkSourceGutterRendererLines *renderer)
 
 	view = gtk_source_gutter_renderer_get_view (GTK_SOURCE_GUTTER_RENDERER (renderer));
 
-	return gtk_text_view_get_buffer (view);
+	return view != NULL ? gtk_text_view_get_buffer (view) : NULL;
 }
 
 static void
@@ -103,35 +94,75 @@ static void
 gutter_renderer_change_buffer (GtkSourceGutterRenderer *renderer,
                                GtkTextBuffer           *old_buffer)
 {
-	GtkTextView *view;
-	GtkSourceGutterRendererLines *lines;
+	GtkSourceGutterRendererLines *lines = GTK_SOURCE_GUTTER_RENDERER_LINES (renderer);
+	GtkTextBuffer *buffer;
 
-	lines = GTK_SOURCE_GUTTER_RENDERER_LINES (renderer);
-
-	if (old_buffer)
+	if (old_buffer != NULL)
 	{
-		g_signal_handler_disconnect (old_buffer,
-		                             lines->priv->changed_handler_id);
+		g_signal_handlers_disconnect_by_func (old_buffer,
+						      on_buffer_changed,
+						      lines);
 	}
 
-	view = gtk_source_gutter_renderer_get_view (renderer);
+	buffer = get_buffer (lines);
 
-	if (view)
+	if (buffer != NULL)
 	{
-		GtkTextBuffer *buffer;
+		g_signal_connect_object (buffer,
+					 "changed",
+					 G_CALLBACK (on_buffer_changed),
+					 lines,
+					 0);
 
-		buffer = gtk_text_view_get_buffer (view);
+		recalculate_size (lines);
+	}
 
-		if (buffer)
-		{
-			lines->priv->changed_handler_id =
-				g_signal_connect (buffer,
-				                  "changed",
-				                  G_CALLBACK (on_buffer_changed),
-				                  lines);
+	lines->priv->prev_line_num = 0;
 
-			recalculate_size (lines);
-		}
+	if (GTK_SOURCE_GUTTER_RENDERER_CLASS (gtk_source_gutter_renderer_lines_parent_class)->change_buffer != NULL)
+	{
+		GTK_SOURCE_GUTTER_RENDERER_CLASS (gtk_source_gutter_renderer_lines_parent_class)->change_buffer (renderer,
+														 old_buffer);
+	}
+}
+
+static void
+on_view_style_updated (GtkTextView                  *view,
+		       GtkSourceGutterRendererLines *renderer)
+{
+	/* Force to recalculate the size. */
+	renderer->priv->num_line_digits = -1;
+	recalculate_size (renderer);
+}
+
+static void
+gutter_renderer_change_view (GtkSourceGutterRenderer *renderer,
+			     GtkTextView             *old_view)
+{
+	GtkTextView *new_view;
+
+	if (old_view != NULL)
+	{
+		g_signal_handlers_disconnect_by_func (old_view,
+						      on_view_style_updated,
+						      renderer);
+	}
+
+	new_view = gtk_source_gutter_renderer_get_view (renderer);
+
+	if (new_view != NULL)
+	{
+		g_signal_connect_object (new_view,
+					 "style-updated",
+					 G_CALLBACK (on_view_style_updated),
+					 renderer,
+					 0);
+	}
+
+	if (GTK_SOURCE_GUTTER_RENDERER_CLASS (gtk_source_gutter_renderer_lines_parent_class)->change_view != NULL)
+	{
+		GTK_SOURCE_GUTTER_RENDERER_CLASS (gtk_source_gutter_renderer_lines_parent_class)->change_view (renderer,
+													       old_view);
 	}
 }
 
@@ -166,6 +197,77 @@ gutter_renderer_query_data (GtkSourceGutterRenderer      *renderer,
 	g_free (text);
 }
 
+static gint
+get_last_visible_line_number (GtkSourceGutterRendererLines *lines)
+{
+	GtkTextView *view;
+	GdkRectangle visible_rect;
+	GtkTextIter iter;
+
+	view = gtk_source_gutter_renderer_get_view (GTK_SOURCE_GUTTER_RENDERER (lines));
+
+	gtk_text_view_get_visible_rect (view, &visible_rect);
+
+	gtk_text_view_get_line_at_y (view,
+				     &iter,
+				     visible_rect.y + visible_rect.height,
+				     NULL);
+
+	gtk_text_iter_forward_line (&iter);
+
+	return gtk_text_iter_get_line (&iter);
+}
+
+static void
+gutter_renderer_end (GtkSourceGutterRenderer *renderer)
+{
+	GtkSourceGutterRendererLines *lines = GTK_SOURCE_GUTTER_RENDERER_LINES (renderer);
+	GtkTextBuffer *buffer = get_buffer (lines);
+
+	if (buffer != NULL)
+	{
+		gint line_num = get_last_visible_line_number (lines);
+
+		/* When the text is modified in a GtkTextBuffer, GtkTextView tries to
+		 * redraw the smallest required region. But the information displayed in
+		 * the gutter may become invalid in a bigger region.
+		 * See https://bugzilla.gnome.org/show_bug.cgi?id=732418 for an example
+		 * where line numbers are not updated correctly when splitting a wrapped
+		 * line.
+		 * The performances should not be a big problem here. Correctness is
+		 * more important than performances. It just triggers a second
+		 * draw.
+		 * The queue_draw() is called in gutter_renderer_end(), because
+		 * the first draw is anyway needed to avoid flickering (if the
+		 * first draw is not done, there will be a white region in the
+		 * gutter during one frame).
+		 * Another solution that has better performances is to compare
+		 * the total number of lines in the buffer, instead of the last
+		 * visible line. But it has the drawback that the gutter is
+		 * continuously redrawn during file loading.
+		 *
+		 * FIXME A better solution would be to add a vfunc in the
+		 * GutterRenderer so that the Gutter can ask each renderer for
+		 * the invalidation region, before drawing. So that only one
+		 * draw is needed, and the solution would be more generic (if
+		 * other renderers also need a different invalidation region
+		 * than the GtkTextView). But the GutterRendererClass doesn't
+		 * have padding for future expansion, so it must wait for
+		 * GtkSourceView 4.
+		 */
+		if (lines->priv->prev_line_num != line_num)
+		{
+			lines->priv->prev_line_num = line_num;
+			gtk_source_gutter_renderer_queue_draw (renderer);
+		}
+	}
+
+	if (GTK_SOURCE_GUTTER_RENDERER_CLASS (gtk_source_gutter_renderer_lines_parent_class)->end != NULL)
+	{
+		GTK_SOURCE_GUTTER_RENDERER_CLASS (gtk_source_gutter_renderer_lines_parent_class)->end (renderer);
+	}
+}
+
 static void
 extend_selection_to_line (GtkSourceGutterRendererLines *renderer,
                           GtkTextIter                  *line_start)
@@ -280,27 +382,24 @@ gutter_renderer_query_activatable (GtkSourceGutterRenderer *renderer,
 static void
 gtk_source_gutter_renderer_lines_class_init (GtkSourceGutterRendererLinesClass *klass)
 {
-	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 	GtkSourceGutterRendererClass *renderer_class = GTK_SOURCE_GUTTER_RENDERER_CLASS (klass);
 
-	object_class->finalize = gtk_source_gutter_renderer_lines_finalize;
-
 	renderer_class->query_data = gutter_renderer_query_data;
+	renderer_class->end = gutter_renderer_end;
 	renderer_class->query_activatable = gutter_renderer_query_activatable;
 	renderer_class->activate = gutter_renderer_activate;
 	renderer_class->change_buffer = gutter_renderer_change_buffer;
-
-	g_type_class_add_private (object_class, sizeof(GtkSourceGutterRendererLinesPrivate));
+	renderer_class->change_view = gutter_renderer_change_view;
 }
 
 static void
 gtk_source_gutter_renderer_lines_init (GtkSourceGutterRendererLines *self)
 {
-	self->priv = GTK_SOURCE_GUTTER_RENDERER_LINES_GET_PRIVATE (self);
+	self->priv = gtk_source_gutter_renderer_lines_get_instance_private (self);
 }
 
 GtkSourceGutterRenderer *
-gtk_source_gutter_renderer_lines_new ()
+gtk_source_gutter_renderer_lines_new (void)
 {
 	return g_object_new (GTK_SOURCE_TYPE_GUTTER_RENDERER_LINES, NULL);
 }
