@@ -19,11 +19,20 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-
 #include "gtksourcegutterrendererpixbuf.h"
 #include "gtksourceview-i18n.h"
 #include "gtksourcepixbufhelper.h"
 
+/**
+ * SECTION:gutterrendererpixbuf
+ * @Short_description: Renders a pixbuf in the gutter
+ * @Title: GtkSourceGutterRendererPixbuf
+ * @See_also: #GtkSourceGutterRenderer, #GtkSourceGutter
+ *
+ * A #GtkSourceGutterRendererPixbuf can be used to render an image in a cell of
+ * #GtkSourceGutter.
+ */
+
 struct _GtkSourceGutterRendererPixbufPrivate
 {
 	GtkSourcePixbufHelper *helper;
@@ -71,28 +80,31 @@ gutter_renderer_pixbuf_draw (GtkSourceGutterRenderer      *renderer,
                              GtkTextIter                  *end,
                              GtkSourceGutterRendererState  state)
 {
-	GtkSourceGutterRendererPixbuf *pix;
+	GtkSourceGutterRendererPixbuf *pix = GTK_SOURCE_GUTTER_RENDERER_PIXBUF (renderer);
 	gint width;
 	gint height;
 	gfloat xalign;
 	gfloat yalign;
 	GtkSourceGutterRendererAlignmentMode mode;
 	GtkTextView *view;
+	gint scale;
 	gint x = 0;
 	gint y = 0;
 	GdkPixbuf *pixbuf;
+	cairo_surface_t *surface;
 
 	/* Chain up to draw background */
-	GTK_SOURCE_GUTTER_RENDERER_CLASS (
-		gtk_source_gutter_renderer_pixbuf_parent_class)->draw (renderer,
-		                                                       cr,
-		                                                       background_area,
-		                                                       cell_area,
-		                                                       start,
-		                                                       end,
-		                                                       state);
+	if (GTK_SOURCE_GUTTER_RENDERER_CLASS (gtk_source_gutter_renderer_pixbuf_parent_class)->draw != NULL)
+	{
+		GTK_SOURCE_GUTTER_RENDERER_CLASS (gtk_source_gutter_renderer_pixbuf_parent_class)->draw (renderer,
+													 cr,
+													 background_area,
+													 cell_area,
+													 start,
+													 end,
+													 state);
+	}
 
-	pix = GTK_SOURCE_GUTTER_RENDERER_PIXBUF (renderer);
 	view = gtk_source_gutter_renderer_get_view (renderer);
 
 	pixbuf = gtk_source_pixbuf_helper_render (pix->priv->helper,
@@ -105,7 +117,23 @@ gutter_renderer_pixbuf_draw (GtkSourceGutterRenderer      *renderer,
 	}
 
 	width = gdk_pixbuf_get_width (pixbuf);
-	height = gdk_pixbuf_get_width (pixbuf);
+	height = gdk_pixbuf_get_height (pixbuf);
+
+	/*
+	 * We might have gotten a pixbuf back from the helper that will allow
+	 * us to render for HiDPI. If we detect this, we pretend that we got a
+	 * different size back and then gdk_cairo_surface_create_from_pixbuf()
+	 * will take care of the rest.
+	 */
+	scale = gtk_widget_get_scale_factor (GTK_WIDGET (view));
+	if ((scale > 1) &&
+	    ((width > cell_area->width) || (height > cell_area->height)) &&
+	    (width <= (cell_area->width * scale)) &&
+	    (height <= (cell_area->height * scale)))
+	{
+		width = width / scale;
+		height = height / scale;
+	}
 
 	gtk_source_gutter_renderer_get_alignment (renderer,
 	                                          &xalign,
@@ -141,10 +169,16 @@ gutter_renderer_pixbuf_draw (GtkSourceGutterRenderer      *renderer,
 			           &x,
 			           &y);
 			break;
+		default:
+			g_assert_not_reached ();
 	}
 
-	gdk_cairo_set_source_pixbuf (cr, pixbuf, x, y);
+	surface = gdk_cairo_surface_create_from_pixbuf (pixbuf, scale, NULL);
+	cairo_set_source_surface (cr, surface, x, y);
+
 	cairo_paint (cr);
+
+	cairo_surface_destroy (surface);
 }
 
 static void
@@ -286,8 +320,8 @@ gtk_source_gutter_renderer_pixbuf_class_init (GtkSourceGutterRendererPixbufClass
 	g_object_class_install_property (object_class,
 	                                 PROP_PIXBUF,
 	                                 g_param_spec_object ("pixbuf",
-	                                                      _("Pixbuf"),
-	                                                      _("The pixbuf"),
+	                                                      "Pixbuf",
+	                                                      "The pixbuf",
 	                                                      GDK_TYPE_PIXBUF,
 	                                                      G_PARAM_READWRITE));
 
@@ -301,24 +335,24 @@ gtk_source_gutter_renderer_pixbuf_class_init (GtkSourceGutterRendererPixbufClass
 	g_object_class_install_property (object_class,
 	                                 PROP_STOCK_ID,
 	                                 g_param_spec_string ("stock-id",
-	                                                      _("Stock Id"),
-	                                                      _("The stock id"),
+	                                                      "Stock Id",
+	                                                      "The stock id",
 	                                                      NULL,
 	                                                      G_PARAM_READWRITE | G_PARAM_DEPRECATED));
 
 	g_object_class_install_property (object_class,
 	                                 PROP_ICON_NAME,
 	                                 g_param_spec_string ("icon-name",
-	                                                      _("Icon Name"),
-	                                                      _("The icon name"),
+	                                                      "Icon Name",
+	                                                      "The icon name",
 	                                                      NULL,
 	                                                      G_PARAM_READWRITE));
 
 	g_object_class_install_property (object_class,
 	                                 PROP_GICON,
 	                                 g_param_spec_object ("gicon",
-	                                                      _("GIcon"),
-	                                                      _("The gicon"),
+	                                                      "GIcon",
+	                                                      "The gicon",
 	                                                      G_TYPE_ICON,
 	                                                      G_PARAM_READWRITE));
 }
@@ -340,7 +374,7 @@ gtk_source_gutter_renderer_pixbuf_init (GtkSourceGutterRendererPixbuf *self)
  *
  **/
 GtkSourceGutterRenderer *
-gtk_source_gutter_renderer_pixbuf_new ()
+gtk_source_gutter_renderer_pixbuf_new (void)
 {
 	return g_object_new (GTK_SOURCE_TYPE_GUTTER_RENDERER_PIXBUF, NULL);
 }
