@@ -38,11 +38,10 @@
 #include "gtksourceview-i18n.h"
 #include "gtksourcelanguage-private.h"
 #include "gtksourcelanguage.h"
-#include "gtksourceview-marshal.h"
 
 /**
  * SECTION:language
- * @Short_description: Object representing a syntax highlighted language
+ * @Short_description: Represents a syntax highlighted language
  * @Title: GtkSourceLanguage
  * @See_also: #GtkSourceLanguageManager
  *
@@ -225,32 +224,32 @@ gtk_source_language_class_init (GtkSourceLanguageClass *klass)
 	g_object_class_install_property (object_class,
 					 PROP_ID,
 					 g_param_spec_string ("id",
-						 	      _("Language id"),
-							      _("Language id"),
+						 	      "Language id",
+							      "Language id",
 							      NULL,
 							      G_PARAM_READABLE));
 
 	g_object_class_install_property (object_class,
 					 PROP_NAME,
 					 g_param_spec_string ("name",
-						 	      _("Language name"),
-							      _("Language name"),
+						 	      "Language name",
+							      "Language name",
 							      NULL,
 							      G_PARAM_READABLE));
 
 	g_object_class_install_property (object_class,
 					 PROP_SECTION,
 					 g_param_spec_string ("section",
-						 	      _("Language section"),
-							      _("Language section"),
+						 	      "Language section",
+							      "Language section",
 							      NULL,
 							      G_PARAM_READABLE));
 
 	g_object_class_install_property (object_class,
 					 PROP_HIDDEN,
 					 g_param_spec_boolean ("hidden",
-							       _("Hidden"),
-							       _("Whether the language should be hidden from the user"),
+							       "Hidden",
+							       "Whether the language should be hidden from the user",
 							       FALSE,
 							       G_PARAM_READABLE));
 }
@@ -549,8 +548,9 @@ gtk_source_language_get_hidden (GtkSourceLanguage *language)
  * @language: a #GtkSourceLanguage.
  * @name: metadata property name.
  *
- * Returns: value of property @name stored in the metadata of @language
- * or %NULL if language doesn't contain that metadata property.
+ * Returns: (nullable) (transfer none): value of property @name stored in
+ * the metadata of @language or %NULL if language does not contain the
+ * specified metadata property.
  * The returned string is owned by @language and should not be freed
  * or modified.
  **/
@@ -573,9 +573,9 @@ gtk_source_language_get_metadata (GtkSourceLanguage *language,
  * retrieve the "mimetypes" metadata property and split it into an
  * array.
  *
- * Returns: (array zero-terminated=1) (transfer full): a newly-allocated
- * %NULL terminated array containing the mime types or %NULL if no
- * mime types are found.
+ * Returns: (nullable) (array zero-terminated=1) (transfer full):
+ * a newly-allocated %NULL terminated array containing the mime types
+ * or %NULL if no mime types are found.
  * The returned array must be freed with g_strfreev().
  **/
 gchar **
@@ -600,8 +600,9 @@ gtk_source_language_get_mime_types (GtkSourceLanguage *language)
  * an utility wrapper around gtk_source_language_get_metadata() to
  * retrieve the "globs" metadata property and split it into an array.
  *
- * Returns: (array zero-terminated=1) (transfer full): a newly-allocated
- * %NULL terminated array containing the globs or %NULL if no globs are found.
+ * Returns: (nullable) (array zero-terminated=1) (transfer full):
+ * a newly-allocated %NULL terminated array containing the globs or %NULL
+ * if no globs are found.
  * The returned array must be freed with g_strfreev().
  **/
 gchar **
@@ -834,9 +835,9 @@ force_styles (GtkSourceLanguage *language)
  *
  * Returns the ids of the styles defined by this @language.
  *
- * Returns: (array zero-terminated=1) (transfer full): a  %NULL terminated
- * array containing ids of the styles defined by this @language or
- * %NULL if no style is defined.
+ * Returns: (nullable) (array zero-terminated=1) (transfer full):
+ * a newly-allocated %NULL terminated array containing ids of the
+ * styles defined by this @language or %NULL if no style is defined.
  * The returned array must be freed with g_strfreev().
 */
 gchar **
@@ -873,10 +874,10 @@ get_style_info (GtkSourceLanguage *language, const char *style_id)
  *
  * Returns the name of the style with ID @style_id defined by this @language.
  *
- * Returns: the name of the style with ID @style_id defined by this @language or
- * %NULL if the style has no name or there is no style with ID @style_id defined
- * by this @language. The returned string is owned by the @language and must
- * not be modified.
+ * Returns: (nullable) (transfer none): the name of the style with ID @style_id
+ * defined by this @language or %NULL if the style has no name or there is no
+ * style with ID @style_id defined by this @language.
+ * The returned string is owned by the @language and must not be modified.
  */
 const gchar *
 gtk_source_language_get_style_name (GtkSourceLanguage *language,
@@ -901,9 +902,9 @@ gtk_source_language_get_style_name (GtkSourceLanguage *language,
  * Returns the ID of the style to use if the specified @style_id
  * is not present in the current style scheme.
  *
- * Returns: the ID of the style to use if the specified @style_id
- * is not present in the current style scheme or %NULL if the style has
- * no fallback defined.
+ * Returns: (nullable) (transfer none): the ID of the style to use if the
+ * specified @style_id is not present in the current style scheme or %NULL
+ * if the style has no fallback defined.
  * The returned string is owned by the @language and must not be modified.
  *
  * Since: 3.4
