@@ -24,7 +24,6 @@
 #include "gtksourcemark.h"
 #include "gtksourceview-i18n.h"
 #include "gtksourcepixbufhelper.h"
-#include "gtksourceview-marshal.h"
 
 /**
  * SECTION:markattributes
@@ -285,8 +284,8 @@ gtk_source_mark_attributes_class_init (GtkSourceMarkAttributesClass *klass)
 	g_object_class_install_property (object_class,
 	                                 PROP_BACKGROUND,
 	                                 g_param_spec_boxed ("background",
-	                                                     _("Background"),
-	                                                     _("The background"),
+	                                                     "Background",
+	                                                     "The background",
 	                                                     GDK_TYPE_RGBA,
 	                                                     G_PARAM_READWRITE));
 
@@ -300,10 +299,10 @@ gtk_source_mark_attributes_class_init (GtkSourceMarkAttributesClass *klass)
 	g_object_class_install_property (object_class,
 	                                 PROP_STOCK_ID,
 	                                 g_param_spec_string ("stock-id",
-	                                                      _("Stock Id"),
-	                                                      _("The stock id"),
+	                                                      "Stock Id",
+	                                                      "The stock id",
 	                                                      NULL,
-	                                                      G_PARAM_READWRITE));
+	                                                      G_PARAM_READWRITE | G_PARAM_DEPRECATED));
 
 	/**
 	 * GtkSourceMarkAttributes:pixbuf:
@@ -313,8 +312,8 @@ gtk_source_mark_attributes_class_init (GtkSourceMarkAttributesClass *klass)
 	g_object_class_install_property (object_class,
 	                                 PROP_PIXBUF,
 	                                 g_param_spec_object ("pixbuf",
-	                                                      _("Pixbuf"),
-	                                                      _("The pixbuf"),
+	                                                      "Pixbuf",
+	                                                      "The pixbuf",
 	                                                      GDK_TYPE_PIXBUF,
 	                                                      G_PARAM_READWRITE));
 
@@ -326,8 +325,8 @@ gtk_source_mark_attributes_class_init (GtkSourceMarkAttributesClass *klass)
 	g_object_class_install_property (object_class,
 	                                 PROP_ICON_NAME,
 	                                 g_param_spec_string ("icon-name",
-	                                                      _("Icon Name"),
-	                                                      _("The icon name"),
+	                                                      "Icon Name",
+	                                                      "The icon name",
 	                                                      NULL,
 	                                                      G_PARAM_READWRITE));
 
@@ -339,8 +338,8 @@ gtk_source_mark_attributes_class_init (GtkSourceMarkAttributesClass *klass)
 	g_object_class_install_property (object_class,
 	                                 PROP_GICON,
 	                                 g_param_spec_object ("gicon",
-	                                                      _("GIcon"),
-	                                                      _("The GIcon"),
+	                                                      "GIcon",
+	                                                      "The GIcon",
 	                                                      G_TYPE_ICON,
 	                                                      G_PARAM_READWRITE));
 
@@ -360,9 +359,7 @@ gtk_source_mark_attributes_class_init (GtkSourceMarkAttributesClass *klass)
 		              G_TYPE_FROM_CLASS (klass),
 		              G_SIGNAL_RUN_LAST,
 		              0,
-		              NULL,
-		              NULL,
-		              _gtksourceview_marshal_STRING__OBJECT,
+		              NULL, NULL, NULL,
 		              G_TYPE_STRING,
 		              1,
 		              GTK_SOURCE_TYPE_MARK);
@@ -383,9 +380,7 @@ gtk_source_mark_attributes_class_init (GtkSourceMarkAttributesClass *klass)
 		              G_TYPE_FROM_CLASS (klass),
 		              G_SIGNAL_RUN_LAST,
 		              0,
-		              NULL,
-		              NULL,
-		              _gtksourceview_marshal_STRING__OBJECT,
+		              NULL, NULL, NULL,
 		              G_TYPE_STRING,
 		              1,
 		              GTK_SOURCE_TYPE_MARK);
@@ -407,7 +402,7 @@ gtk_source_mark_attributes_init (GtkSourceMarkAttributes *self)
  * Returns: (transfer full): a new source mark attributes.
  */
 GtkSourceMarkAttributes *
-gtk_source_mark_attributes_new ()
+gtk_source_mark_attributes_new (void)
 {
 	return g_object_new (GTK_SOURCE_TYPE_MARK_ATTRIBUTES, NULL);
 }
