@@ -38,7 +38,7 @@
 
 /**
  * SECTION:printcompositor
- * @Short_description: Compose a #GtkSourceBuffer for printing
+ * @Short_description: Compose a GtkSourceBuffer for printing
  * @Title: GtkSourcePrintCompositor
  *
  * The #GtkSourcePrintCompositor object is used to compose a #GtkSourceBuffer
@@ -442,8 +442,8 @@ gtk_source_print_compositor_class_init (GtkSourcePrintCompositorClass *klass)
 	g_object_class_install_property (object_class,
 					 PROP_BUFFER,
 					 g_param_spec_object ("buffer",
-							      _("Source Buffer"),
-							      _("The GtkSourceBuffer object to print"),
+							      "Source Buffer",
+							      "The GtkSourceBuffer object to print",
 							      GTK_SOURCE_TYPE_BUFFER,
 							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
@@ -460,8 +460,8 @@ gtk_source_print_compositor_class_init (GtkSourcePrintCompositorClass *klass)
 	g_object_class_install_property (object_class,
 					 PROP_TAB_WIDTH,
 					 g_param_spec_uint ("tab-width",
-							    _("Tab Width"),
-							    _("Width of a tab character expressed in spaces"),
+							    "Tab Width",
+							    "Width of a tab character expressed in spaces",
 							    1,
 							    MAX_TAB_WIDTH,
 							    DEFAULT_TAB_WIDTH,
@@ -481,10 +481,8 @@ gtk_source_print_compositor_class_init (GtkSourcePrintCompositorClass *klass)
 	g_object_class_install_property (object_class,
 					 PROP_WRAP_MODE,
 					 g_param_spec_enum ("wrap-mode",
-							    _("Wrap Mode"),
-							    _("Whether to wrap lines never, "
-							      "at word boundaries, or at "
-							      "character boundaries."),
+							    "Wrap Mode",
+							    "",
 							    GTK_TYPE_WRAP_MODE,
 							    GTK_WRAP_NONE,
 							    G_PARAM_READWRITE));
@@ -502,10 +500,8 @@ gtk_source_print_compositor_class_init (GtkSourcePrintCompositorClass *klass)
 	g_object_class_install_property (object_class,
 					 PROP_HIGHLIGHT_SYNTAX,
 					 g_param_spec_boolean ("highlight-syntax",
-							       _("Highlight Syntax"),
-							       _("Whether to print the "
-								 "document with highlighted "
-								 "syntax"),
+							       "Highlight Syntax",
+							       "",
 							       TRUE,
 							       G_PARAM_READWRITE));
 
@@ -524,9 +520,8 @@ gtk_source_print_compositor_class_init (GtkSourcePrintCompositorClass *klass)
 	g_object_class_install_property (object_class,
 					 PROP_PRINT_LINE_NUMBERS,
 					 g_param_spec_uint ("print-line-numbers",
-							    _("Print Line Numbers"),
-							    _("Interval of printed line numbers "
-							      "(0 means no numbers)"),
+							    "Print Line Numbers",
+							    "",
 							    0, 100, 1,
 							    G_PARAM_READWRITE));
 
@@ -547,9 +542,8 @@ gtk_source_print_compositor_class_init (GtkSourcePrintCompositorClass *klass)
 	g_object_class_install_property (object_class,
 					 PROP_PRINT_HEADER,
 					 g_param_spec_boolean ("print-header",
-							       _("Print Header"),
-							       _("Whether to print a header "
-								 "in each page"),
+							       "Print Header",
+							       "",
 							       FALSE,
 							       G_PARAM_READWRITE));
 
@@ -570,9 +564,8 @@ gtk_source_print_compositor_class_init (GtkSourcePrintCompositorClass *klass)
 	g_object_class_install_property (object_class,
 					 PROP_PRINT_FOOTER,
 					 g_param_spec_boolean ("print-footer",
-							       _("Print Footer"),
-							       _("Whether to print a footer "
-								 "in each page"),
+							       "Print Footer",
+							       "",
 							       FALSE,
 							       G_PARAM_READWRITE));
 
@@ -593,9 +586,8 @@ gtk_source_print_compositor_class_init (GtkSourcePrintCompositorClass *klass)
 	g_object_class_install_property (object_class,
 					 PROP_BODY_FONT_NAME,
 					 g_param_spec_string ("body-font-name",
-							      _("Body Font Name"),
-							      _("Name of the font to use for the text body "
-							        "(e.g. \"Monospace 10\")"),
+							      "Body Font Name",
+							      "",
 							      NULL,
 							      G_PARAM_READWRITE));
 
@@ -617,9 +609,8 @@ gtk_source_print_compositor_class_init (GtkSourcePrintCompositorClass *klass)
 	g_object_class_install_property (object_class,
 					 PROP_LINE_NUMBERS_FONT_NAME,
 					 g_param_spec_string ("line-numbers-font-name",
-							      _("Line Numbers Font Name"),
-							      _("Name of the font to use for the line numbers "
-							        "(e.g. \"Monospace 10\")"),
+							      "Line Numbers Font Name",
+							      "",
 							      NULL,
 							      G_PARAM_READWRITE));
 
@@ -641,9 +632,8 @@ gtk_source_print_compositor_class_init (GtkSourcePrintCompositorClass *klass)
 	g_object_class_install_property (object_class,
 					 PROP_HEADER_FONT_NAME,
 					 g_param_spec_string ("header-font-name",
-							      _("Header Font Name"),
-							      _("Name of the font to use for the page header "
-							        "(e.g. \"Monospace 10\")"),
+							      "Header Font Name",
+							      "",
 							      NULL,
 							      G_PARAM_READWRITE));
 
@@ -665,9 +655,8 @@ gtk_source_print_compositor_class_init (GtkSourcePrintCompositorClass *klass)
 	g_object_class_install_property (object_class,
 					 PROP_FOOTER_FONT_NAME,
 					 g_param_spec_string ("footer-font-name",
-							      _("Footer Font Name"),
-							      _("Name of the font to use for the page footer "
-							        "(e.g. \"Monospace 10\")"),
+							      "Footer Font Name",
+							      "",
 							      NULL,
 							      G_PARAM_READWRITE));
 
@@ -682,10 +671,8 @@ gtk_source_print_compositor_class_init (GtkSourcePrintCompositorClass *klass)
 	g_object_class_install_property (object_class,
 					 PROP_N_PAGES,
 					 g_param_spec_int ("n-pages",
-							   _("Number of pages"),
-							   _("The number of pages in the document "
-							     "(-1 means the document has not been "
-							     "completely paginated)."),
+							   "Number of pages",
+							   "",
 							   -1, G_MAXINT, -1,
 							   G_PARAM_READABLE));
 }
@@ -816,7 +803,7 @@ gtk_source_print_compositor_new_from_view (GtkSourceView *view)
 	font_desc = pango_context_get_font_description (pango_context);
 
 	compositor->priv->body_font = pango_font_description_copy (font_desc);
-	g_object_notify (G_OBJECT (compositor), "body-font-name"); // FIXME: is this needed?
+	g_object_notify (G_OBJECT (compositor), "body-font-name"); /* FIXME: is this needed? */
 
 	return compositor;
 }
@@ -1137,8 +1124,11 @@ gtk_source_print_compositor_get_print_footer (GtkSourcePrintCompositor *composit
  * Sets strftime like header format strings, to be printed on the
  * left, center and right of the top of each page.  The strings may
  * include strftime(3) codes which will be expanded at print time.
- * All strftime() codes are accepted, with the addition of #N for the
- * page number and #Q for the page count.
+ * A subset of strftime() codes are accepted, see g_date_time_format()
+ * for more details on the accepted format specifiers.
+ * Additionally the following format specifiers are accepted:
+ * - #N: the page number
+ * - #Q: the page count.
  *
  * @separator specifies if a solid line should be drawn to separate
  * the header from the document text.
@@ -1185,24 +1175,8 @@ gtk_source_print_compositor_set_header_format (GtkSourcePrintCompositor *composi
  * @center: (allow-none): a format string to print on the center of the footer.
  * @right: (allow-none): a format string to print on the right of the footer.
  *
- * Sets strftime like header format strings, to be printed on the
- * left, center and right of the bottom of each page. The strings may
- * include strftime(3) codes which will be expanded at print time.
- * All strftime() codes are accepted, with the addition of #N for the
- * page number and #Q for the page count.
- *
- * @separator specifies if a solid line should be drawn to separate
- * the footer from the document text.
- *
- * If %NULL is given for any of the three arguments, that particular
- * string will not be printed.
- *
- * For the footer to be printed, in
- * addition to specifying format strings, you need to enable footer
- * printing with gtk_source_print_compositor_set_print_footer().
- *
- * This function cannot be called anymore after the first call to the
- * gtk_source_print_compositor_paginate() function.
+ * See gtk_source_print_compositor_set_header_format() for more information
+ * about the parameters.
  *
  * Since: 2.2
  **/
@@ -1852,6 +1826,8 @@ setup_pango_layouts (GtkSourcePrintCompositor *compositor,
 			 */
 			pango_layout_set_ellipsize (layout, PANGO_ELLIPSIZE_END);
 			break;
+		default:
+			g_return_if_reached ();
 	}
 
 	set_layout_tab_width (compositor, layout);
@@ -1902,95 +1878,16 @@ setup_pango_layouts (GtkSourcePrintCompositor *compositor,
 	}
 }
 
-/* ---- Header and footer ---- */
-
-/* Most of this code taken from GLib's g_date_strftime() in gdate.c
- * GLIB - Library of useful routines for C programming
- * Copyright (C) 1995-1997  Peter Mattis, Spencer Kimball and Josh MacDonald */
-
-static gchar *
-strdup_strftime (const gchar *format, const struct tm *tm)
-{
-	gsize locale_format_len = 0;
-	gchar *locale_format;
-	gsize tmplen;
-	gchar *tmpbuf;
-	gsize tmpbufsize;
-	gchar *convbuf;
-	gsize convlen = 0;
-	GError *error = NULL;
-
-	g_return_val_if_fail (format != NULL, NULL);
-	g_return_val_if_fail (tm != NULL, NULL);
-
-	locale_format = g_locale_from_utf8 (format, -1, NULL, &locale_format_len, &error);
-
-	if (error)
-	{
-		g_warning (G_STRLOC "Error converting format to locale encoding: %s",
-			   error->message);
-		g_error_free (error);
-
-		return NULL;
-	}
-
-	tmpbufsize = MAX (128, locale_format_len * 2);
-	while (TRUE)
-	{
-		tmpbuf = g_malloc (tmpbufsize);
-
-		/* Set the first byte to something other than '\0', to be able to
-		 * recognize whether strftime actually failed or just returned "".
-		 */
-		tmpbuf[0] = '\1';
-		tmplen = strftime (tmpbuf, tmpbufsize, locale_format, tm);
-
-		if (tmplen == 0 && tmpbuf[0] != '\0')
-		{
-			g_free (tmpbuf);
-			tmpbufsize *= 2;
-
-			if (tmpbufsize > 65536)
-			{
-				g_warning (G_STRLOC "Maximum buffer size for strdup_strftime "
-					   "exceeded: giving up");
-				g_free (locale_format);
-				return NULL;
-			}
-		}
-		else
-			break;
-	}
-	g_free (locale_format);
-
-	convbuf = g_locale_to_utf8 (tmpbuf, tmplen, NULL, &convlen, &error);
-	g_free (tmpbuf);
-
-	if (error)
-	{
-		g_warning (G_STRLOC "Error converting results of strftime to UTF-8: %s",
-			   error->message);
-		g_error_free (error);
-
-		return NULL;
-	}
-
-	return convbuf;
-}
-
 static gchar *
 evaluate_format_string (GtkSourcePrintCompositor *compositor,
 			const gchar              *format)
 {
+	GDateTime *now;
 	GString *eval;
 	gchar *eval_str, *retval;
-	const struct tm *tm;
-	time_t now;
 	gunichar ch;
 
-	/* get time */
-	time (&now);
-	tm = localtime (&now);
+	now = g_date_time_new_now_local ();
 
 	/* analyze format string and replace the codes we know */
 	eval = g_string_new_len (NULL, strlen (format));
@@ -2021,9 +1918,11 @@ evaluate_format_string (GtkSourcePrintCompositor *compositor,
 	}
 
 	eval_str = g_string_free (eval, FALSE);
-	retval = strdup_strftime (eval_str, tm);
+	retval = g_date_time_format (now, eval_str);
 	g_free (eval_str);
 
+	g_date_time_unref (now);
+
 	return retval;
 }
 
@@ -2312,18 +2211,32 @@ get_iter_attrs (GtkSourcePrintCompositor *compositor,
 		if (bg_set)
 		{
 			GdkRGBA *color = NULL;
-			if (bg) pango_attribute_destroy (bg);
+
+			if (bg != NULL)
+			{
+				pango_attribute_destroy (bg);
+			}
+
 			g_object_get (tag, "background-rgba", &color, NULL);
-			bg = pango_attr_background_new (color->red, color->green, color->blue);
+			bg = pango_attr_background_new (color->red * 65535,
+							color->green * 65535,
+							color->blue * 65535);
 			gdk_rgba_free (color);
 		}
 
 		if (fg_set)
 		{
 			GdkRGBA *color = NULL;
-			if (fg) pango_attribute_destroy (fg);
+
+			if (fg != NULL)
+			{
+				pango_attribute_destroy (fg);
+			}
+
 			g_object_get (tag, "foreground-rgba", &color, NULL);
-			fg = pango_attr_foreground_new (color->red, color->green, color->blue);
+			fg = pango_attr_foreground_new (color->red * 65535,
+							color->green * 65535,
+							color->blue * 65535);
 			gdk_rgba_free (color);
 		}
 
@@ -2695,7 +2608,7 @@ gtk_source_print_compositor_paginate (GtkSourcePrintCompositor *compositor,
 					double layout_line_height;
 
 					pango_layout_iter_get_line_extents (layout_iter, NULL, &logical_rect);
-					layout_line_height = logical_rect.height / PANGO_SCALE;
+					layout_line_height = (double) logical_rect.height / PANGO_SCALE;
 
 					if (is_first_line &&
 					    line_is_numbered (compositor, line_number))
