@@ -24,49 +24,60 @@
 #define __GTK_SOURCE_REGEX_H__
 
 #include <glib.h>
-#include <gtksourceview/gtksourcetypes.h>
+#include "gtksourcetypes-private.h"
 
 G_BEGIN_DECLS
 
+G_GNUC_INTERNAL
 GtkSourceRegex	*_gtk_source_regex_new		(const gchar         *pattern,
 						 GRegexCompileFlags   flags,
 						 GError             **error);
 
+G_GNUC_INTERNAL
 GtkSourceRegex	*_gtk_source_regex_ref		(GtkSourceRegex *regex);
 
+G_GNUC_INTERNAL
 void		 _gtk_source_regex_unref	(GtkSourceRegex *regex);
 
+G_GNUC_INTERNAL
 GtkSourceRegex	*_gtk_source_regex_resolve	(GtkSourceRegex *regex,
 						 GtkSourceRegex *start_regex,
 						 const gchar    *matched_text);
 
+G_GNUC_INTERNAL
 gboolean	 _gtk_source_regex_is_resolved	(GtkSourceRegex *regex);
 
+G_GNUC_INTERNAL
 gboolean	_gtk_source_regex_match		(GtkSourceRegex *regex,
 						 const gchar    *line,
 						 gint             byte_length,
 						 gint             byte_pos);
 
+G_GNUC_INTERNAL
 gchar		*_gtk_source_regex_fetch	(GtkSourceRegex *regex,
 						 gint            num);
 
+G_GNUC_INTERNAL
 void		 _gtk_source_regex_fetch_pos	(GtkSourceRegex *regex,
 						 const gchar    *text,
 						 gint            num,
 						 gint           *start_pos, /* character offsets */
 						 gint           *end_pos);  /* character offsets */
 
+G_GNUC_INTERNAL
 void		 _gtk_source_regex_fetch_pos_bytes (GtkSourceRegex *regex,
 						    gint            num,
 						    gint           *start_pos_p, /* byte offsets */
 						    gint           *end_pos_p);  /* byte offsets */
 
+G_GNUC_INTERNAL
 void		 _gtk_source_regex_fetch_named_pos (GtkSourceRegex *regex,
 						    const gchar    *text,
 						    const gchar    *name,
 						    gint           *start_pos, /* character offsets */
 						    gint           *end_pos);  /* character offsets */
 
+G_GNUC_INTERNAL
 const gchar	*_gtk_source_regex_get_pattern	(GtkSourceRegex *regex);
 
 G_END_DECLS
