@@ -27,6 +27,13 @@
 
 G_BEGIN_DECLS
 
+/*
+ * We need to be lower than the application priority to allow
+ * application overrides. And we need enough room for
+ * GtkSourceMap to be able to override the style priority.
+ */
+#define GTK_SOURCE_STYLE_PROVIDER_PRIORITY (GTK_STYLE_PROVIDER_PRIORITY_APPLICATION-2)
+
 enum
 {
 	GTK_SOURCE_STYLE_USE_LINE_BACKGROUND = 1 << 0,	/*< nick=use_line_background >*/
@@ -35,7 +42,9 @@ enum
 	GTK_SOURCE_STYLE_USE_ITALIC          = 1 << 3,	/*< nick=use_italic >*/
 	GTK_SOURCE_STYLE_USE_BOLD            = 1 << 4,	/*< nick=use_bold >*/
 	GTK_SOURCE_STYLE_USE_UNDERLINE       = 1 << 5,	/*< nick=use_underline >*/
-	GTK_SOURCE_STYLE_USE_STRIKETHROUGH   = 1 << 6	/*< nick=use_strikethrough >*/
+	GTK_SOURCE_STYLE_USE_STRIKETHROUGH   = 1 << 6,	/*< nick=use_strikethrough >*/
+	GTK_SOURCE_STYLE_USE_SCALE           = 1 << 7,	/*< nick=use_scale >*/
+	GTK_SOURCE_STYLE_USE_UNDERLINE_COLOR = 1 << 8	/*< nick=use_underline_color >*/
 };
 
 struct _GtkSourceStyle
@@ -48,10 +57,13 @@ struct _GtkSourceStyle
 	const gchar *foreground;
 	const gchar *background;
 	const gchar *line_background;
+	const gchar *scale;
+	const gchar *underline_color;
+
+	PangoUnderline underline;
 
 	guint italic : 1;
 	guint bold : 1;
-	guint underline : 1;
 	guint strikethrough : 1;
 	guint mask : 12;
 };
