@@ -372,6 +372,29 @@ gtk_source_style_scheme_get_filename (GtkSourceStyleScheme *scheme)
 	return scheme->priv->filename;
 }
 
+/*
+ * Try to parse a color string.
+ * If the color can be parsed, return the offset in the string
+ * with the real start of the color (either the string itself, or after
+ * the initial '#' character).
+ */
+static const gchar *
+color_parse (const gchar *color,
+             GdkRGBA     *rgba)
+{
+	if ((*color == '#') && gdk_rgba_parse (rgba, color + 1))
+	{
+		return color + 1;
+	}
+
+	if (gdk_rgba_parse (rgba, color))
+	{
+		return color;
+	}
+
+	return NULL;
+}
+
 /*
  * get_color_by_name:
  * @scheme: a #GtkSourceStyleScheme.
@@ -396,15 +419,9 @@ get_color_by_name (GtkSourceStyleScheme *scheme,
 	{
 		GdkRGBA dummy;
 
-		if (gdk_rgba_parse (&dummy, name + 1))
-		{
-			color = name + 1;
-		}
-		else if (gdk_rgba_parse (&dummy, name))
-		{
-			color = name;
-		}
-		else
+		color = color_parse (name, &dummy);
+
+		if (color == NULL)
 		{
 			g_warning ("could not parse color '%s'", name);
 		}
@@ -440,7 +457,8 @@ fix_style_colors (GtkSourceStyleScheme *scheme,
 	} attributes[] = {
 		{ GTK_SOURCE_STYLE_USE_BACKGROUND, G_STRUCT_OFFSET (GtkSourceStyle, background) },
 		{ GTK_SOURCE_STYLE_USE_FOREGROUND, G_STRUCT_OFFSET (GtkSourceStyle, foreground) },
-		{ GTK_SOURCE_STYLE_USE_LINE_BACKGROUND, G_STRUCT_OFFSET (GtkSourceStyle, line_background) }
+		{ GTK_SOURCE_STYLE_USE_LINE_BACKGROUND, G_STRUCT_OFFSET (GtkSourceStyle, line_background) },
+		{ GTK_SOURCE_STYLE_USE_UNDERLINE_COLOR, G_STRUCT_OFFSET (GtkSourceStyle, underline_color) }
 	};
 
 	style = gtk_source_style_copy (real_style);
@@ -581,7 +599,7 @@ get_color (GtkSourceStyle *style,
 
 		if (style->mask & mask)
 		{
-			if (color == NULL || !gdk_rgba_parse (dest, color))
+			if (color == NULL || !color_parse (color, dest))
 			{
 				g_warning ("%s: invalid color '%s'", G_STRLOC,
 					   color != NULL ? color : "(null)");
@@ -652,18 +670,21 @@ get_cursors_css_style (GtkSourceStyleScheme *scheme,
 	if (!get_color (secondary_style, TRUE, &secondary_color))
 	{
 		GtkStyleContext *context;
+		GdkRGBA *rgba;
 
 		context = gtk_widget_get_style_context (widget);
 
 		gtk_style_context_get (context,
 				       GTK_STATE_FLAG_NORMAL,
-				       "background-color", &secondary_color,
+				       "background-color", &rgba,
 				       NULL);
 
 		/* shade the secondary cursor */
-		secondary_color.red *= 0.5;
-		secondary_color.green *= 0.5;
-		secondary_color.blue *= 0.5;
+		secondary_color.red = rgba->red * 0.5;
+		secondary_color.green = rgba->green * 0.5;
+		secondary_color.blue = rgba->blue * 0.5;
+
+		gdk_rgba_free (rgba);
 	}
 
 	primary_color_str = gdk_rgba_to_string (&primary_color);
@@ -743,7 +764,7 @@ _gtk_source_style_scheme_apply (GtkSourceStyleScheme *scheme,
 	context = gtk_widget_get_style_context (GTK_WIDGET (widget));
 	gtk_style_context_add_provider (context,
 	                                GTK_STYLE_PROVIDER (scheme->priv->css_provider),
-	                                GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+	                                GTK_SOURCE_STYLE_PROVIDER_PRIORITY);
 
 	G_GNUC_BEGIN_IGNORE_DEPRECATIONS;
 	/* See https://bugzilla.gnome.org/show_bug.cgi?id=708583 */
@@ -762,7 +783,7 @@ _gtk_source_style_scheme_apply (GtkSourceStyleScheme *scheme,
 	{
 		gtk_style_context_add_provider (context,
 						GTK_STYLE_PROVIDER (scheme->priv->css_provider_cursors),
-						GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+						GTK_SOURCE_STYLE_PROVIDER_PRIORITY);
 
 		G_GNUC_BEGIN_IGNORE_DEPRECATIONS;
 		gtk_style_context_invalidate (context);
@@ -921,6 +942,14 @@ generate_css_style (GtkSourceStyleScheme *scheme)
 	g_string_free (final_style, TRUE);
 }
 
+static gboolean
+parse_bool (char *value)
+{
+	return (g_ascii_strcasecmp (value, "true") == 0 ||
+	        g_ascii_strcasecmp (value, "yes") == 0 ||
+	        g_ascii_strcasecmp (value, "1") == 0);
+}
+
 static void
 get_bool (xmlNode    *node,
 	  const char *propname,
@@ -933,9 +962,7 @@ get_bool (xmlNode    *node,
 	if (tmp != NULL)
 	{
 		*mask |= mask_value;
-		*value = g_ascii_strcasecmp ((char*) tmp, "true") == 0 ||
-			 g_ascii_strcasecmp ((char*) tmp, "yes") == 0 ||
-			 g_ascii_strcasecmp ((char*) tmp, "1") == 0;
+		*value = parse_bool ((char*) tmp);
 	}
 
 	xmlFree (tmp);
@@ -957,8 +984,9 @@ parse_style (GtkSourceStyleScheme *scheme,
 	guint mask = 0;
 	gboolean bold = FALSE;
 	gboolean italic = FALSE;
-	gboolean underline = FALSE;
 	gboolean strikethrough = FALSE;
+	xmlChar *underline = NULL;
+	xmlChar *underline_color = NULL;
 	xmlChar *scale = NULL;
 	xmlChar *tmp;
 
@@ -1004,13 +1032,20 @@ parse_style (GtkSourceStyleScheme *scheme,
 	line_bg = xmlGetProp (node, BAD_CAST "line-background");
 	get_bool (node, "italic", &mask, GTK_SOURCE_STYLE_USE_ITALIC, &italic);
 	get_bool (node, "bold", &mask, GTK_SOURCE_STYLE_USE_BOLD, &bold);
-	get_bool (node, "underline", &mask, GTK_SOURCE_STYLE_USE_UNDERLINE, &underline);
 	get_bool (node, "strikethrough", &mask, GTK_SOURCE_STYLE_USE_STRIKETHROUGH, &strikethrough);
+	underline = xmlGetProp (node, BAD_CAST "underline");
+	underline_color = xmlGetProp (node, BAD_CAST "underline-color");
 	scale = xmlGetProp (node, BAD_CAST "scale");
 
 	if (use_style)
 	{
-		if (fg != NULL || bg != NULL || line_bg != NULL || mask != 0 || scale != NULL)
+		if (fg != NULL ||
+		    bg != NULL ||
+		    line_bg != NULL ||
+		    mask != 0 ||
+		    underline != NULL ||
+		    underline_color != NULL ||
+		    scale != NULL)
 		{
 			g_set_error (error, ERROR_QUARK, 0,
 				     "in style '%s': style attributes used along with use-style",
@@ -1020,6 +1055,9 @@ parse_style (GtkSourceStyleScheme *scheme,
 			xmlFree (fg);
 			xmlFree (bg);
 			xmlFree (line_bg);
+			xmlFree (underline);
+			xmlFree (underline_color);
+			xmlFree (scale);
 			return FALSE;
 		}
 
@@ -1033,7 +1071,6 @@ parse_style (GtkSourceStyleScheme *scheme,
 		result->mask = mask;
 		result->bold = bold;
 		result->italic = italic;
-		result->underline = underline;
 		result->strikethrough = strikethrough;
 
 		if (fg != NULL)
@@ -1053,6 +1090,45 @@ parse_style (GtkSourceStyleScheme *scheme,
 			result->line_background = g_intern_string ((char*) line_bg);
 			result->mask |= GTK_SOURCE_STYLE_USE_LINE_BACKGROUND;
 		}
+
+		if (underline != NULL)
+		{
+			/* Up until 3.16 underline was a "bool", so for backward
+			 * compat we accept underline="true" and map it to "single"
+			 */
+			if (parse_bool ((char *) underline))
+			{
+				result->underline = PANGO_UNDERLINE_SINGLE;
+				result->mask |= GTK_SOURCE_STYLE_USE_UNDERLINE;
+			}
+			else
+			{
+				GEnumClass *enum_class;
+				GEnumValue *enum_value;
+				gchar *underline_lowercase;
+
+				enum_class = G_ENUM_CLASS (g_type_class_ref (PANGO_TYPE_UNDERLINE));
+
+				underline_lowercase = g_ascii_strdown ((char*) underline, -1);
+				enum_value = g_enum_get_value_by_nick (enum_class, underline_lowercase);
+				g_free (underline_lowercase);
+
+				if (enum_value != NULL)
+				{
+					result->underline = enum_value->value;
+					result->mask |= GTK_SOURCE_STYLE_USE_UNDERLINE;
+				}
+
+				g_type_class_unref (enum_class);
+			}
+		}
+
+		if (underline_color != NULL)
+		{
+			result->underline_color = g_intern_string ((char*) underline_color);
+			result->mask |= GTK_SOURCE_STYLE_USE_UNDERLINE_COLOR;
+		}
+
 		if (scale != NULL)
 		{
 			result->scale = g_intern_string ((char*) scale);
@@ -1063,10 +1139,12 @@ parse_style (GtkSourceStyleScheme *scheme,
 	*style_p = result;
 	*style_name_p = style_name;
 
-	xmlFree (scale);
-	xmlFree (line_bg);
-	xmlFree (bg);
 	xmlFree (fg);
+	xmlFree (bg);
+	xmlFree (line_bg);
+	xmlFree (underline);
+	xmlFree (underline_color);
+	xmlFree (scale);
 
 	return TRUE;
 }
