@@ -54,10 +54,6 @@ struct _GtkSourceStyleSchemeClass
 
 GType			 gtk_source_style_scheme_get_type			(void) G_GNUC_CONST;
 
-G_GNUC_INTERNAL
-GtkSourceStyleScheme	*_gtk_source_style_scheme_new				(const gchar          *id,
-										 const gchar          *name);
-
 const gchar             *gtk_source_style_scheme_get_id				(GtkSourceStyleScheme *scheme);
 
 const gchar             *gtk_source_style_scheme_get_name			(GtkSourceStyleScheme *scheme);
@@ -104,6 +100,11 @@ GtkSourceStyle          *_gtk_source_style_scheme_get_draw_spaces_style		(GtkSou
 G_GNUC_INTERNAL
 gboolean		 _gtk_source_style_scheme_get_current_line_color	(GtkSourceStyleScheme *scheme,
 										 GdkRGBA              *color);
+
+G_GNUC_INTERNAL
+gboolean		 _gtk_source_style_scheme_get_background_pattern_color	(GtkSourceStyleScheme *scheme,
+										 GdkRGBA              *color);
+
 G_END_DECLS
 
 #endif  /* __GTK_SOURCE_STYLE_SCHEME_H__ */
