@@ -21,14 +21,13 @@
 
 #include "gtksourcestyleschememanager.h"
 #include "gtksourcestylescheme.h"
-#include "gtksourceview-marshal.h"
 #include "gtksourceview-i18n.h"
 #include "gtksourceview-utils.h"
 #include <string.h>
 
 /**
  * SECTION:styleschememanager
- * @Short_description: Object which provides access to #GtkSourceStyleScheme<!-- -->s
+ * @Short_description: Provides access to GtkSourceStyleSchemes
  * @Title: GtkSourceStyleSchemeManager
  * @See_also: #GtkSourceStyleScheme
  *
@@ -150,18 +149,16 @@ gtk_source_style_scheme_manager_class_init (GtkSourceStyleSchemeManagerClass *kl
 	g_object_class_install_property (object_class,
 					 PROP_SEARCH_PATH,
 					 g_param_spec_boxed ("search-path",
-						 	     _("Style scheme search path"),
-							     _("List of directories and files where the "
-							       "style schemes are located"),
+						 	     "Style scheme search path",
+							     "List of directories and files where the style schemes are located",
 							     G_TYPE_STRV,
 							     G_PARAM_READWRITE));
 
 	g_object_class_install_property (object_class,
 					 PROP_SCHEME_IDS,
 					 g_param_spec_boxed ("scheme-ids",
-						 	     _("Scheme ids"),
-							     _("List of the ids of the available "
-							       "style schemes"),
+						 	     "Scheme ids",
+							     "List of the ids of the available style schemes",
 							     G_TYPE_STRV,
 							     G_PARAM_READABLE));
 }
@@ -544,9 +541,9 @@ gtk_source_style_scheme_manager_force_rescan (GtkSourceStyleSchemeManager *manag
  *
  * Returns the ids of the available style schemes.
  *
- * Returns: (array zero-terminated=1) (transfer none): a %NULL-terminated array
- * of string containing the ids of the available style schemes or %NULL if no
- * style scheme is available.
+ * Returns: (nullable) (array zero-terminated=1) (transfer none):
+ * a %NULL-terminated array of strings containing the ids of the available
+ * style schemes or %NULL if no style scheme is available.
  * The array is sorted alphabetically according to the scheme name.
  * The array is owned by the @manager and must not be modified.
  */
