@@ -19,15 +19,8 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
-
-#include <glib.h>
-
 #include "gtktextregion.h"
 
-
 #undef ENABLE_DEBUG
 /*
 #define ENABLE_DEBUG
@@ -123,7 +116,7 @@ gtk_text_region_new (GtkTextBuffer *buffer)
 
 	g_return_val_if_fail (GTK_IS_TEXT_BUFFER (buffer), NULL);
 
-	region = g_new (GtkTextRegion, 1);
+	region = g_slice_new (GtkTextRegion);
 	region->subregions = NULL;
 	region->time_stamp = 0;
 
@@ -146,7 +139,7 @@ gtk_text_region_destroy (GtkTextRegion *region)
 			gtk_text_buffer_delete_mark (region->buffer, sr->start);
 			gtk_text_buffer_delete_mark (region->buffer, sr->end);
 		}
-		g_free (sr);
+		g_slice_free (Subregion, sr);
 		region->subregions = g_list_delete_link (region->subregions,
 							 region->subregions);
 	}
@@ -159,7 +152,7 @@ gtk_text_region_destroy (GtkTextRegion *region)
 		region->buffer = NULL;
 	}
 
-	g_free (region);
+	g_slice_free (GtkTextRegion, region);
 }
 
 GtkTextBuffer *
@@ -185,7 +178,7 @@ gtk_text_region_clear_zero_length_subregions (GtkTextRegion *region)
 		if (gtk_text_iter_equal (&start, &end)) {
 			gtk_text_buffer_delete_mark (region->buffer, sr->start);
 			gtk_text_buffer_delete_mark (region->buffer, sr->end);
-			g_free (sr);
+			g_slice_free (Subregion, sr);
 			if (node == region->subregions)
 				region->subregions = node = g_list_delete_link (node, node);
 			else
@@ -235,7 +228,7 @@ gtk_text_region_add (GtkTextRegion     *region,
 
 	if (start_node == NULL || end_node == NULL || end_node == start_node->prev) {
 		/* create the new subregion */
-		Subregion *sr = g_new0 (Subregion, 1);
+		Subregion *sr = g_slice_new0 (Subregion);
 		sr->start = gtk_text_buffer_create_mark (region->buffer, NULL, &start, TRUE);
 		sr->end = gtk_text_buffer_create_mark (region->buffer, NULL, &end, FALSE);
 
@@ -266,13 +259,13 @@ gtk_text_region_add (GtkTextRegion     *region,
 				q = l->data;
 				gtk_text_buffer_delete_mark (region->buffer, q->start);
 				gtk_text_buffer_delete_mark (region->buffer, q->end);
-				g_free (q);
+				g_slice_free (Subregion, q);
 				l = g_list_delete_link (l, l);
 			}
 			q = l->data;
 			gtk_text_buffer_delete_mark (region->buffer, q->start);
 			sr->end = q->end;
-			g_free (q);
+			g_slice_free (Subregion, q);
 			l = g_list_delete_link (l, l);
 		}
 		/* now move marks if that action expands the region */
@@ -341,7 +334,7 @@ gtk_text_region_subtract (GtkTextRegion     *region,
 		    !gtk_text_iter_equal (&end, &sr_end_iter)) {
 			/* the ending point is also inside the first
                            subregion: we need to split */
-			Subregion *new_sr = g_new0 (Subregion, 1);
+			Subregion *new_sr = g_slice_new0 (Subregion);
 			new_sr->end = sr->end;
 			new_sr->start = gtk_text_buffer_create_mark (region->buffer,
 								     NULL, &end, TRUE);
@@ -402,7 +395,7 @@ gtk_text_region_subtract (GtkTextRegion     *region,
 			sr = node->data;
 			gtk_text_buffer_delete_mark (region->buffer, sr->start);
 			gtk_text_buffer_delete_mark (region->buffer, sr->end);
-			g_free (sr);
+			g_slice_free (Subregion, sr);
 			region->subregions = g_list_delete_link (region->subregions,
 								 node);
 			node = l;
@@ -495,7 +488,7 @@ gtk_text_region_intersect (GtkTextRegion     *region,
 
 	/* starting node */
 	if (gtk_text_iter_in_range (&start, &sr_start_iter, &sr_end_iter)) {
-		new_sr = g_new0 (Subregion, 1);
+		new_sr = g_slice_new0 (Subregion);
 		new_region->subregions = g_list_prepend (new_region->subregions, new_sr);
 
 		new_sr->start = gtk_text_buffer_create_mark (new_region->buffer, NULL,
@@ -528,7 +521,7 @@ gtk_text_region_intersect (GtkTextRegion     *region,
 							  sr->start);
 			gtk_text_buffer_get_iter_at_mark (region->buffer, &sr_end_iter, sr->end);
 
-			new_sr = g_new0 (Subregion, 1);
+			new_sr = g_slice_new0 (Subregion);
 			new_region->subregions = g_list_prepend (new_region->subregions, new_sr);
 			new_sr->start = gtk_text_buffer_create_mark (new_region->buffer, NULL,
 								     &sr_start_iter, TRUE);
@@ -543,7 +536,7 @@ gtk_text_region_intersect (GtkTextRegion     *region,
 		gtk_text_buffer_get_iter_at_mark (region->buffer, &sr_start_iter, sr->start);
 		gtk_text_buffer_get_iter_at_mark (region->buffer, &sr_end_iter, sr->end);
 
-		new_sr = g_new0 (Subregion, 1);
+		new_sr = g_slice_new0 (Subregion);
 		new_region->subregions = g_list_prepend (new_region->subregions, new_sr);
 
 		new_sr->start = gtk_text_buffer_create_mark (new_region->buffer, NULL,
@@ -620,15 +613,18 @@ gtk_text_region_iterator_next (GtkTextRegionIterator *iter)
 	real = (GtkTextRegionIteratorReal *)iter;
 	g_return_val_if_fail (check_iterator (real), FALSE);
 
-	if (real->subregions != NULL) {
+	if (real->subregions != NULL)
+	{
 		real->subregions = g_list_next (real->subregions);
 		return TRUE;
 	}
 	else
+	{
 		return FALSE;
+	}
 }
 
-void
+gboolean
 gtk_text_region_iterator_get_subregion (GtkTextRegionIterator *iter,
 					GtkTextIter           *start,
 					GtkTextIter           *end)
@@ -636,24 +632,31 @@ gtk_text_region_iterator_get_subregion (GtkTextRegionIterator *iter,
 	GtkTextRegionIteratorReal *real;
 	Subregion *sr;
 
-	g_return_if_fail (iter != NULL);
+	g_return_val_if_fail (iter != NULL, FALSE);
 
 	real = (GtkTextRegionIteratorReal *)iter;
-	g_return_if_fail (check_iterator (real));
-	g_return_if_fail (real->subregions != NULL);
+	g_return_val_if_fail (check_iterator (real), FALSE);
+	g_return_val_if_fail (real->subregions != NULL, FALSE);
 
 	if (real->region->buffer == NULL)
 	{
-		return;
+		return FALSE;
 	}
 
 	sr = (Subregion*)real->subregions->data;
-	g_return_if_fail (sr != NULL);
+	g_return_val_if_fail (sr != NULL, FALSE);
 
-	if (start)
+	if (start != NULL)
+	{
 		gtk_text_buffer_get_iter_at_mark (real->region->buffer, start, sr->start);
-	if (end)
+	}
+
+	if (end != NULL)
+	{
 		gtk_text_buffer_get_iter_at_mark (real->region->buffer, end, sr->end);
+	}
+
+	return TRUE;
 }
 
 void
@@ -669,16 +672,15 @@ gtk_text_region_debug_print (GtkTextRegion *region)
 	}
 
 	g_print ("Subregions: ");
-	l = region->subregions;
-	while (l) {
+
+	for (l = region->subregions; l != NULL; l = l->next)
+	{
 		Subregion *sr = l->data;
 		GtkTextIter iter1, iter2;
 		gtk_text_buffer_get_iter_at_mark (region->buffer, &iter1, sr->start);
 		gtk_text_buffer_get_iter_at_mark (region->buffer, &iter2, sr->end);
 		g_print ("%d-%d ", gtk_text_iter_get_offset (&iter1),
 			 gtk_text_iter_get_offset (&iter2));
-		l = l->next;
 	}
 	g_print ("\n");
 }
-
