@@ -1,8 +1,5 @@
-#include "config.h"
-#include <string.h>
-
 #include <gtk/gtk.h>
-#include <gtksourceview/gtksourceprintcompositor.h>
+#include <gtksourceview/gtksource.h>
 
 static void
 test_buffer_ref (void)
