@@ -52,9 +52,14 @@ struct _TestWidgetPrivate
 	GtkSourceView *view;
 	GtkSourceBuffer *buffer;
 	GtkSourceFile *file;
+	GtkSourceMap *map;
+	GtkCheckButton *show_map_checkbutton;
+	GtkCheckButton *smart_backspace_checkbutton;
 	GtkCheckButton *indent_width_checkbutton;
 	GtkSpinButton *indent_width_spinbutton;
 	GtkLabel *cursor_position_info;
+	GtkSourceStyleSchemeChooserButton *chooser_button;
+	GtkComboBoxText *background_pattern;
 };
 
 GType test_widget_get_type (void);
@@ -915,6 +920,28 @@ add_source_mark_attributes (GtkSourceView *view)
 	g_object_unref (attrs);
 }
 
+static void
+on_background_pattern_changed (GtkComboBox *combobox,
+                               TestWidget  *self)
+{
+	gchar *text;
+
+	text = gtk_combo_box_text_get_active_text (GTK_COMBO_BOX_TEXT (combobox));
+
+	if (g_strcmp0 (text, "Grid") == 0)
+	{
+		gtk_source_view_set_background_pattern (self->priv->view,
+		                                        GTK_SOURCE_BACKGROUND_PATTERN_TYPE_GRID);
+	}
+	else
+	{
+		gtk_source_view_set_background_pattern (self->priv->view,
+		                                        GTK_SOURCE_BACKGROUND_PATTERN_TYPE_NONE);
+	}
+
+	g_free (text);
+}
+
 static void
 test_widget_dispose (GObject *object)
 {
@@ -956,16 +983,19 @@ test_widget_class_init (TestWidgetClass *klass)
 	gtk_widget_class_bind_template_callback (widget_class, smart_home_end_changed_cb);
 
 	gtk_widget_class_bind_template_child_private (widget_class, TestWidget, view);
+	gtk_widget_class_bind_template_child_private (widget_class, TestWidget, map);
+	gtk_widget_class_bind_template_child_private (widget_class, TestWidget, show_map_checkbutton);
+	gtk_widget_class_bind_template_child_private (widget_class, TestWidget, smart_backspace_checkbutton);
 	gtk_widget_class_bind_template_child_private (widget_class, TestWidget, indent_width_checkbutton);
 	gtk_widget_class_bind_template_child_private (widget_class, TestWidget, indent_width_spinbutton);
 	gtk_widget_class_bind_template_child_private (widget_class, TestWidget, cursor_position_info);
+	gtk_widget_class_bind_template_child_private (widget_class, TestWidget, chooser_button);
+	gtk_widget_class_bind_template_child_private (widget_class, TestWidget, background_pattern);
 }
 
 static void
 test_widget_init (TestWidget *self)
 {
-	PangoFontDescription *font_desc;
-
 	self->priv = test_widget_get_instance_private (self);
 
 	gtk_widget_init_template (GTK_WIDGET (self));
@@ -1007,12 +1037,28 @@ test_widget_init (TestWidget *self)
 			  G_CALLBACK (line_mark_activated_cb),
 			  self);
 
-	font_desc = pango_font_description_from_string ("monospace");
-	if (font_desc != NULL)
-	{
-		gtk_widget_override_font (GTK_WIDGET (self->priv->view), font_desc);
-		pango_font_description_free (font_desc);
-	}
+	g_object_bind_property (self->priv->chooser_button,
+	                        "style-scheme",
+	                        self->priv->buffer,
+	                        "style-scheme",
+	                        G_BINDING_SYNC_CREATE);
+
+	g_object_bind_property (self->priv->show_map_checkbutton,
+	                        "active",
+	                        self->priv->map,
+	                        "visible",
+	                        G_BINDING_SYNC_CREATE);
+
+	g_object_bind_property (self->priv->smart_backspace_checkbutton,
+	                        "active",
+	                        self->priv->view,
+	                        "smart-backspace",
+	                        G_BINDING_SYNC_CREATE);
+
+	g_signal_connect (self->priv->background_pattern,
+	                  "changed",
+	                  G_CALLBACK (on_background_pattern_changed),
+	                  self);
 
 	open_file (self, TOP_SRCDIR "/gtksourceview/gtksourcebuffer.c");
 }
